//Board.cpp//

#include "Board.h"
#include <fstream>
#include <iostream>
#include <sstream>

using std::ifstream;
using std::stringstream;
using std::ofstream;
using std::endl;

Board :: Board(const string& rkoStrFileN)
{
  if (rkoStrFileN == "")
  {
    return;
  }
  
  m_oStrFileN = rkoStrFileN;

  ifstream oBFile(m_oStrFileN.c_str());

  // if the file's absent or inaccessible
  if (!oBFile)
  {
    throw BadBoardFile (m_oStrFileN);
  }
  
  // get the portals
  try
  {
    while(oBFile)
    {
      int iSPos, iEPos;
      oBFile >> iSPos >> iEPos;
      AddPortal(iSPos-1,iEPos-1);
    }
  }
  catch (BadPosEx& rErr)
  {
    stringstream oSrm;
    oSrm << "Error building board from file " << m_oStrFileN 
      << " : " << rErr.what() << ": " << rErr.GetPos()+1;
    string oStrErr = oSrm.str();

    throw BuildError(oStrErr);
  }
  catch (BadPortalEx& rErr)
  {
    stringstream oSrm;
    oSrm << "Error building board from file " << m_oStrFileN 
      << " : " << rErr.what() << ": " << rErr.GetSPos()+1 << " -> " 
      << rErr.GetEPos()+1;
    string oStrErr = oSrm.str();

    throw BuildError(oStrErr);
  }
  catch (runtime_error& rErr)
  {
    stringstream oSrm;
    oSrm << "Error building board from file " << m_oStrFileN 
      << " : " << rErr.what();
    string oStrErr = oSrm.str();

    throw BuildError(oStrErr);
  }

  oBFile.close();
}

void Board::Save(const string& rkoStrFileN) const
{
  ofstream oSrmFile(rkoStrFileN.c_str());

  if (!oSrmFile)
  {
    throw BuildError("Could not open file " + rkoStrFileN + " for saving.");
  }
  
  for(int i=0; i<NCELLS; i++)
  {
    if (m_oCell[i].GetType() != NOPORTAL)
    {
      oSrmFile << i+1 << " " <<  m_oCell[i].GetEndPos()+1 << endl;
    }
  }
}

void Board :: AddPortal(int iSPos, int iEPos)
{
  // throw an exception if the starting position is invalid
  if ((iSPos < 0) || (iSPos >= NCELLS))
  {
    throw BadPosEx(iSPos);
  }

  // throw an exception if the end position is invalid
  if ((iEPos < 0) || (iEPos >= NCELLS))
  {
    throw BadPosEx(iEPos);
  }
  
  // if the start position is after the end pos, this is a snake
  if (iSPos > iEPos)
  {
    if (iSPos == 99)
    {
      throw BadPortalEx(iSPos, iEPos);
    }
    if (m_oCell[iEPos].GetType() == LADDER)
    {
      throw BadPortalEx(iSPos, iEPos);
    }
    m_oCell[iSPos].AddPortal(SNAKE,iEPos);
  }
  // if the start position is before the end pos, this is a ladder 
  else if (iSPos < iEPos)
  {
    if (iSPos == 0)
    {
      throw BadPortalEx(iSPos, iEPos);
    }
    if (m_oCell[iEPos].GetType() == SNAKE)
    {
      throw BadPortalEx(iSPos, iEPos);
    }
    m_oCell[iSPos].AddPortal(LADDER,iEPos);
  }
  // this is invalid
  else
  {
    throw BadPortalEx(iSPos, iEPos);
  }
}

void Board :: DelPortal(int iSPos)
{
  // throw an exception if the starting position is invalid
  if ((iSPos < 0) || (iSPos >= NCELLS))
  {
    throw BadPosEx(iSPos);
  }

  m_oCell[iSPos].DelPortal();
}

int Board :: Walk(int iSPos, int iMoves)
{
  // throw an exception if the starting position is invalid
  if ((iSPos < 0) || (iSPos >= NCELLS))
  {
    throw BadPosEx(iSPos);
  }

  int iEPos = iSPos + iMoves;

  //throw an exception if moves are outside required moves to finish the game
  if ((iMoves < 1) || (iMoves > 6))
  {
    throw BadMovesEx(iMoves);
  }
  else if (iEPos >= NCELLS)
  {
    throw OverThrowEx();
  }
  else if (NOPORTAL == m_oCell[iEPos].GetType())
  {
    return iEPos;
  }
  else
  {
    return Warp(iEPos);
  }
}

int Board :: Warp(int iPos)
{
  // throw an exception if the warp position is invalid
  if ((iPos < 0) || (iPos >= NCELLS))
  {
    throw BadPosEx(iPos);
  }
  
  /// validate if this is a portal
  if (NOPORTAL == m_oCell[iPos].GetType())
  {
    return iPos;
  }
  
  int iEPos = m_oCell[iPos].GetEndPos();
  
  if (NOPORTAL == m_oCell[iEPos].GetType())
  {
    return iEPos;
  }
  else
  {
    return Warp(iEPos);
  }
}
                
const Cell& Board :: operator[] (int iPos)
{
  return m_oCell[iPos];
}

//Board.cpp//
