// BoardFactory.cpp //

extern "C" {
#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
}
#include "BoardFactory.h"
#include <iostream>

using std::cerr;
using std::endl;

BoardFactory *BoardFactory::c_poInstance=NULL;

BoardFactory *BoardFactory::Instance(void)
{
  if (NULL == c_poInstance)
  {
    c_poInstance = new BoardFactory();
  }
  return c_poInstance;
}

BoardFactory::BoardFactory(void)
{
  ReadList();
}
  
BoardFactory::~BoardFactory(void)
{
}

Board *BoardFactory::LoadBoard(const string& rkoStrName)
{
  Board *poBoard;
  try
  {
    poBoard = new Board("/etc/slrc/" + rkoStrName);
    return poBoard;
  }
  catch (runtime_error& rErr)
  {
    try
    {
      poBoard = new  Board((string) getenv("HOME")+"/.slrc/" + rkoStrName);
      return poBoard;
    }
    catch (runtime_error& rErr)
    {
      throw BuildError((string) "Board: " + rkoStrName + " can not be loaded");
    }
  }
}

void BoardFactory::SaveBoard(const Board *pkoBoard, const string& rkoStrName)
{
  string oStrFullName = (string) getenv("HOME") + "/.slrc/"+ rkoStrName;
  int iSOff = oStrFullName.size() - 4;
  if ((oStrFullName.size() <= 4) || (oStrFullName.substr(iSOff, 4) != ".map"))
  {
    oStrFullName += ".map";
  }
  pkoBoard->Save(oStrFullName);
}

const vector< string >& BoardFactory::List(void)
{
  return m_oVecStrBoardMap;
}
    
void BoardFactory::ReadList(void)
{
  m_oVecStrBoardMap.clear();
  
  string oStrSysDir("/etc/slrc");
  string oStrUserDir((string) getenv("HOME") + "/.slrc");

  try
  {
    ScanDir(oStrSysDir);
  }
  catch (runtime_error& rErr)
  {
    cerr << rErr.what() << endl;
  }

  try
  {
    ScanDir(oStrUserDir);
  }
  catch (runtime_error& rErr)
  {
    cerr << rErr.what() << " Creating " << oStrUserDir << " ... " << endl;
    mkdir(oStrUserDir.c_str(), 0700);
  }

  if (m_oVecStrBoardMap.size() == 0)
  {
    throw BuildError("No boards found.");
  }
}

void BoardFactory::ScanDir (const string& rkoStrDirName)
{
  DIR *pstDir = opendir(rkoStrDirName.c_str());
  if (pstDir == NULL)
  {
    throw BuildError(rkoStrDirName + " could not be opened.");
  }

  struct dirent *pstDEnt;
  while (pstDEnt = readdir(pstDir))
  {
    string oStrFName = pstDEnt->d_name;
    int iSOff = oStrFName.size() - 4;
    if ((oStrFName.size() > 4) && (oStrFName.substr(iSOff, 4) == ".map"))
    {
      m_oVecStrBoardMap.push_back(oStrFName);
    }
  }
}

void BoardFactory::Delete (const string& rkoStrName)
{
  string oStrFullName = (string) getenv("HOME") + "/.slrc/" + rkoStrName;
  int iStatus = unlink(oStrFullName.c_str());
}

// BoardFactory.cpp //
