// Game.cpp //

#include "Game.h"
#include "BoardFactory.h"
#include <iostream>

using std::cerr;
using std::endl;

Game::Game(void)
{
  m_poBoard = BoardFactory::Instance()->LoadBoard("defboard.map");
  m_bStarted = false;
  m_iCurrPlayer =0;
}

/// add a player to the game
void Game::AddPlayer(char cID, string oStrName)
{
  m_oVecPlayers.push_back(Player(cID, oStrName));
}

void Game::DelPlayer(char cID)
{
  bool bFound = false;
  
  vector< Player >::iterator oIter;
  for (oIter=m_oVecPlayers.begin(); oIter!=m_oVecPlayers.end(); oIter++)
  {
    if ((*oIter).GetPID() == cID)
    {
      bFound = true;
      break;
    }
  }

  if (bFound)
  {
    m_oVecPlayers.erase(oIter);
  }
}
    
void Game::Start (void)
{
  if (m_bStarted)
  {
    throw GameAlreadyOnEx();
  }

  if (m_oVecPlayers.size() < 2)
  {
    throw NotEnoughPlayersEx();
  }

  m_bStarted = true;
  m_iCurrPlayer = 0;
}

void Game::Stop (void)
{
  m_bStarted = false;
  for(int i=0; i<m_oVecPlayers.size(); i++)
  {
    m_oVecPlayers[i].SetCurPos(0);
  }
}

void Game :: Play (int iMoves)
{
  if (!m_bStarted)
  {
    throw GameNotOnEx();
  }
  
  if (iMoves == -1)
  {
    m_oDie.Roll();
    iMoves = m_oDie.GetFaceVal();
  }

  try
  {
    if ((m_oVecPlayers[m_iCurrPlayer].GetCurPos() == 0) && (iMoves != 6))
    {
      throw (GameStartEx());
    }
    int iPos = m_poBoard->Walk(m_oVecPlayers[m_iCurrPlayer].GetCurPos(),iMoves);
  
    m_oVecPlayers[m_iCurrPlayer].SetCurPos(iPos);

#ifdef DEBUG
    cout << m_oVecPlayers[m_iCurrPlayer].GetPName() << " moves " << iMoves 
      << " squares to  Pos " << m_oVecPlayers[m_iCurrPlayer].GetCurPos() + 1 
      << endl << endl;
#endif // DEBUG
  
    if(m_oVecPlayers[m_iCurrPlayer].GetCurPos() == 99)
    { 
      throw (GameOverEx(m_oVecPlayers[m_iCurrPlayer].GetPName(),
          m_oVecPlayers[m_iCurrPlayer].GetPID()));
    }

  }
  catch (BadMovesEx& rErr)
  {
    // ignore
    cerr << rErr.what() << endl;
  }
  catch (OverThrowEx& rErr)
  {
    m_iCurrPlayer = (m_iCurrPlayer + 1) % m_oVecPlayers.size();
    throw;
  }
  catch (GameStartEx& rErr)
  {
    m_iCurrPlayer = (m_iCurrPlayer + 1) % m_oVecPlayers.size();
    throw;
  }
  if (6 != iMoves)
  {
    m_iCurrPlayer = (m_iCurrPlayer + 1) % m_oVecPlayers.size();
  }
}

void Game::Load (const string& rkostrBName)
{
  try
  {
    Board *poBoard = BoardFactory::Instance()->LoadBoard(rkostrBName);
    delete m_poBoard;
    m_poBoard = poBoard;
  }
  catch(BuildError& rErr)
  {
    throw;
  }
}

void Game::LoadEmpty (void)
{
  try
  {
    Board *poBoard = new Board();
    delete m_poBoard;
    m_poBoard = poBoard;
  }
  catch(BuildError& rErr)
  {
    throw;
  }
}

const Player& Game::GetCurrPlayer (void)
{
  return m_oVecPlayers[m_iCurrPlayer];
}

bool Game :: IsStarted(void)
{
  return m_bStarted == true;
}

// Game.cpp //
